function [h,k,error] = advection_pbc(m,operator)
%
% Solve u_t + au_x = 0  on [ax,bx] with periodic boundary conditions,
% using the Lax-Wendroff method with m interior points.
%
% Returns k, h, and the max-norm of the error.
% This routine can be embedded in a loop on m to test the accuracy,
% perhaps with calls to error_table and/or error_loglog.
%
% From  http://www.amath.washington.edu/~rjl/fdmbook/  (2007)

%operator: +Compact fourth order
%          -Traditional fourth order
global a
a = 1;           % advection velocity

clf              % clear graphics

ax = 0;
bx = 1;
tfinal = 2;                % final time
kt=4;
method=0; % 0: SSP-RK; 1:Carpenter
%operator=-1;
h = (bx-ax)/(m+1);         % h = delta x
k = 1*h                  % time step
nu = a*k;
%/h;                % Courant number
x = linspace(ax,bx,m+2)';  % note x(1)=0 and x(m+2)=1
                           % With periodic BC's there are m+1 unknowns u(2:m+2)
I = 1:(m+1);   % indices of unknowns
% [P,Q,IP,D]=Diff_CompFD77451_e1(m+1)
if operator >0
 [P,Q,IP,D]=Diff_cpt4(m+1);
else
 [P,Q,IP,D]=Diff_trad4(m+1);
end
nsteps = round(tfinal / k)    % number of time steps
nplot = nsteps/100;       % plot solution every nplot time steps
                  % (set nplot=2 to plot every 2 time steps, etc.)
%nplot = nsteps;  % only plot at final time

if method==1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 RK_para_a(1) =              0.0d0;
    RK_para_a(2) =  -567301805773.0d0/ 1357537059087.0d0;
    RK_para_a(3) = -2404267990393.0d0/ 2016746695238.0d0;
    RK_para_a(4) = -3550918686646.0d0/ 2091501179385.0d0;
    RK_para_a(5) = -1275806237668.0d0/  842570457699.0d0;
%    !
    RK_para_b(1) =  1432997174477.0d0/ 9575080441755.0d0;
    RK_para_b(2) =  5161836677717.0d0/13612068292357.0d0;
    RK_para_b(3) =  1720146321549.0d0/ 2090206949498.0d0;
    RK_para_b(4) =  3134564353537.0d0/ 4481467310338.0d0;
    RK_para_b(5) =  2277821191437.0d0/14882151754819.0d0;
%     !
%     RK_para_c(1) =              0.0d0
%     RK_para_c(2) =  1432997174477.0d0/ 9575080441755.0d0
%     RK_para_c(3) =  2526269341429.0d0/ 6820363962896.0d0
%     RK_para_c(4) =  2006345519317.0d0/ 3224310063776.0d0
%     RK_para_c(5) =  2802321613138.0d0/ 2924317926251.0d0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coefficients for SSP-RK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

else
    alpha=coefb(kt);
   % cbin=coefbin(kt);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if abs(k*nsteps - tfinal) > 1e-5
   % The last step won't go exactly to tfinal.
   disp(' ')
   disp(sprintf('WARNING *** k does not divide tfinal, k = %9.5e',k))
   disp(' ')
   end

% initial conditions:
tn = 0;
u0 = eta(x);
u = u0;
u_tmp=u;
% periodic boundary conditions:
%u(1) = u(m+2);   % copy value from rightmost unknown to ghost cell on left
%u(m+3) = u(2);   % copy value from leftmost unknown to ghost cell on right

% initial data on fine grid for plotting:
xfine = linspace(ax,bx,1001)';
ufine = utrue(xfine,0);

% plot initial data:
plot(x,u0,'b.-', xfine,ufine,'r')
axis([0 1 -1.2 1.2])
legend('computed','true')
title('Initial data at time = 0')

input('Hit <return> to continue  ');

% main time-stepping loop:

for n = 1:nsteps
     tnp = tn + k;   % = t_{n+1}

%      % Lax-Wendroff:
%      u(I) = u(I) - 0.5*nu*(u(I+1) - u(I-1)) + ...
%                    0.5*nu^2 * (u(I-1) - 2*u(I) + u(I+1));
% 
%      % periodic boundary conditions:
%      u(1) = u(m+2);   % copy value from rightmost unknown to ghost cell on left
%      u(m+3) = u(2);   % copy value from leftmost unknown to ghost cell on right

%u(I)=u(I)-nu*D*u(I);
 %     u(1) = u(m+2);   % copy value from rightmost unknown to ghost cell on left
         %u_tmp(1:m+2)=0;
         if method==1
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                
      for ks=1:5     
      dudt=-D*u(I);
      u_tmp(I) = RK_para_a(ks)*u_tmp(I) + k*dudt;
      u(I)=u(I)+ RK_para_b(ks)*u_tmp(I);


       end    % plot results at desired times:
           u(m+2) = u(1);  
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
   %begin of SSP-RK
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
else
        temp3=alpha(1)*u(I);
            
        for i=1:kt-2
        temp2=-D*u(I);
        u(I)=u(I)+k*temp2;
        temp3=temp3+alpha(i+1)*u(I);
        end
            
        temp2=-D*u(I);
        u(I)=u(I)+k*temp2;
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            temp2=-D*u(I);
        u(I)=u(I)+k*temp2;
        
        temp3=temp3+alpha(kt)*u(I);
        u(I)=temp3;
    u(m+2) = u(1);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %end of SSPRK
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    end
%                   temp2=-D*u(I);
%         u(I)=u(I)+k*temp2; 
%             u(1) = u(m+2);
       
       
     if mod(n,nplot)==0 | n==nsteps
        uint = u(1:m+2);  % points on the interval (drop ghost cell on right)
        ufine = utrue(xfine,tnp);
        plot(x,uint,'b.-', xfine,ufine,'r')
        axis([0 1 -1.2 1.2])
        title(sprintf('t = %9.5e  after %4i time steps with %5i grid points',...
                       tnp,n,m+1))
        error = max(abs(uint-utrue(x,tnp)));
        %disp(sprintf('at time t = %9.5e  max error =  %9.5e',tnp,error))     
        drawnow
        %if n<nsteps, input('Hit <return> to continue  '); end;

        end

     tn = tnp;   % for next time step
     end
disp(sprintf('at time t = %9.5e  max error =  %9.5e',tnp,error))     
return
%--------------------------------------------------------

function utrue = utrue(x,t)
% true solution for comparison
global a

% For periodic BC's, we need the periodic extension of eta(x).
% Map x-a*t back to unit interval:

xat = rem(x - a*t, 1);
ineg = find(xat<0);
xat(ineg) = xat(ineg) + 1;
utrue = eta(xat);
return


%--------------------------------------------------------

function eta = eta(x)
% initial data
l=4;
beta = 600;
eta = exp(-beta*(x - 0.5).^2);
eta=sin(l*pi*x);

eta=zeros(size(x));
	
for i=1:size(x,1)
   if (x(i)>=.4) & (x(i)<=.6)
      % eta(i)=(x(i)-.6)^2*(x(i)-.4)^2*10^4;
              eta(i)=(x(i)-.6)^4*(x(i)-.4)^4*10^8;
   else 
	eta(i)=0;
   end
  %eta(i)=sin(l*pi*x(i));
end
return


